==> Finance-Ultron-0.3.7.tar.gz: Finance-Ultron-0.3.7/ultron/strategy/experimental/single_factor.py <==
from tokenize import group
from ultron.factor.experimental.normalize import rolling_groups
from ultron.factor.analysis.quantile_analysis import er_quantile_analysis
from ultron.factor.data.quantile import quantile
import numpy as np
import pandas as pd
import pdb, copy
class SingleFactor(object):
    def __init__(self, factor_data, market_data, codes, columns):
        self._factor_data = factor_data
        self._market_data = market_data
        self._codes = codes
        self._columns = columns

    def returns(self, market_data, period):
        price_tb = market_data['closePrice'].unstack()
        price_tb.fillna(method='pad', inplace=True)
        return_tb = np.log(price_tb.shift(-period) / price_tb)
        return_tb = return_tb.replace([np.inf, -np.inf], np.nan)
        return_tb = return_tb.stack().reindex(market_data.index)
        return_tb.name = 'nxt1_ret'
        return return_tb.reset_index()

    def normalize(self, factor_data, windows, columns):
        normalize_data = factor_data.set_index('code').groupby(level=['code']).apply(
                lambda x: rolling_groups(x,columns, windows))
        normalize_data = normalize_data.reset_index().drop(['level_1'],axis=1)
        return normalize_data

    def quantile_analysis(self, normalize_data, factor_name, n_bins):
        df = pd.DataFrame(columns=['q' + str(i) for i in range(1, n_bins+1)])
        grouped = normalize_data.groupby('trade_date')
        for k, g in grouped:
            er = g[factor_name].values
            dx_return = g['nxt1_ret'].values
            res = er_quantile_analysis(er, n_bins=n_bins, dx_return=dx_return, de_trend=True)
            df.loc[k, :] = res
        df['q'] = df['q'+str(n_bins)]-df['q1']
        return df
    
    def quantile(self, normalize_data, factor_name, n_bins):
        grouped = normalize_data.groupby('trade_date')
        res = []
        for k, g in grouped:
            o = copy.deepcopy(g)
            o['group'] = quantile(o[factor_name], n_bins) + 1
            res.append(o.set_index(['trade_date','code']))
        group_dt = pd.concat(res,axis=0)
        return group_dt





    def run(self, columns=[], windows=10, period=1, n_bins=5,
            normalize_data=None, returns_data=None):
        print('start single factor analysis...')
        if len(columns) == 0:
            columns = self._columns
        if normalize_data is None:
            print('start data normalize...')
            normalize_data = self.normalize(factor_data=self._factor_data, windows=windows, 
                            columns=columns)
        if returns_data is None:
            print('start data returns...')
            returns_data = self.returns(self._market_data.set_index(['trade_date','code']), 
                            period)
        
        normalize_data['trade_date'] = pd.to_datetime(normalize_data['trade_date'])
        returns_data['trade_date'] = pd.to_datetime(returns_data['trade_date'])
        total_data = normalize_data.merge(returns_data, on=['trade_date','code'])

        qt_res = []
        for col in columns:
            print("start {0} quantile ...".format(col))
            df = self.quantile(normalize_data[['code','trade_date', col]], col, n_bins)
            qt_res.append({'name':col, "qdata":df})

        qa_res = []
        for col in columns:
            print("start {0} quantile analysis...".format(col))
            df = self.quantile_analysis(normalize_data=total_data, 
                    factor_name=col, n_bins=n_bins)
            qa_res.append({'name':col, "qdata":df})
        return qt_res, qa_res

==> Finance-Ultron-0.3.8.tar.gz: Finance-Ultron-0.3.8/ultron/strategy/experimental/single_factor.py <==
import code
from tokenize import group
from ultron.factor.experimental.normalize import rolling_groups
from ultron.factor.analysis.quantile_analysis import er_quantile_analysis
from ultron.factor.data.quantile import quantile
import numpy as np
import pandas as pd
import pdb, copy
class SingleFactor(object):
    def __init__(self, factor_data, market_data, codes, columns):
        self._factor_data = factor_data
        self._market_data = market_data
        self._codes = codes
        self._columns = columns

    def returns(self, market_data, period):
        price_tb = market_data['closePrice'].unstack()
        price_tb.fillna(method='pad', inplace=True)
        return_tb = np.log(price_tb.shift(-period) / price_tb)
        return_tb = return_tb.replace([np.inf, -np.inf], np.nan)
        return_tb = return_tb.stack().reindex(market_data.index)
        return_tb.name = 'nxt1_ret'
        return return_tb.reset_index()

    def normalize(self, factor_data, windows, columns):
        normalize_data = factor_data.set_index('code').groupby(level=['code']).apply(
                lambda x: rolling_groups(x,columns, windows))
        normalize_data = normalize_data.reset_index().drop(['level_1'],axis=1)
        return normalize_data

    def quantile_analysis(self, normalize_data, factor_name, n_bins):
        df = pd.DataFrame(columns=['q' + str(i) for i in range(1, n_bins+1)])
        grouped = normalize_data.groupby('trade_date')
        for k, g in grouped:
            er = g[factor_name].values
            dx_return = g['nxt1_ret'].values
            res = er_quantile_analysis(er, n_bins=n_bins, dx_return=dx_return, de_trend=True)
            df.loc[k, :] = res
        df['q'] = df['q'+str(n_bins)]-df['q1']
        return df
    
    def quantile(self, normalize_data, factor_name, n_bins):
        grouped = normalize_data.groupby('trade_date')
        res = []
        for k, g in grouped:
            o = copy.deepcopy(g)
            o['group'] = quantile(o[factor_name], n_bins) + 1
            res.append(o.set_index(['trade_date','code']))
        group_dt = pd.concat(res,axis=0)
        return group_dt





    def run(self, codes=None, columns=[], windows=10, period=1, n_bins=5, 
            normalize_data=None, returns_data=None):
        print('start single factor analysis...')
        if len(columns) == 0:
            columns = self._columns
        if normalize_data is None:
            print('start data normalize...')
            normalize_data = self.normalize(factor_data=self._factor_data, windows=windows, 
                            columns=columns)
        if returns_data is None:
            print('start data returns...')
            returns_data = self.returns(self._market_data.set_index(['trade_date','code']), 
                            period)
        if codes is None:
            codes = self._codes
            
        normalize_data['trade_date'] = pd.to_datetime(normalize_data['trade_date'])
        returns_data['trade_date'] = pd.to_datetime(returns_data['trade_date'])
        total_data = normalize_data.merge(returns_data, on=['trade_date','code'])
        
        ### 指定品种
        codes = list(set(total_data['code'].unique().tolist()) & set(codes))
        total_data = total_data.set_index('code').loc[codes].reset_index()
        qt_res = []
        for col in columns:
            print("start {0} quantile ...".format(col))
            df = self.quantile(normalize_data[['code','trade_date', col]], col, n_bins)
            qt_res.append({'name':col, "qdata":df})

        qa_res = []
        for col in columns:
            print("start {0} quantile analysis...".format(col))
            df = self.quantile_analysis(normalize_data=total_data, 
                    factor_name=col, n_bins=n_bins)
            qa_res.append({'name':col, "qdata":df})
        return qt_res, qa_res
